@@ -199,10 +199,14 @@ class NetworkcacheClient(object):
       config = dict(parser.items('networkcache'))
     self.config = config
     path = config.get('signature-private-key-file')
+    pubkey = None
     if path:
       with open(path) as f:
         self.signature_private_key = crypto.load_privatekey(crypto.FILETYPE_PEM,
                                                             f.read())
+      if config.get('download-dir-url'):
+        pubkey = crypto.dump_publickey(crypto.FILETYPE_PEM,
+                                       self.signature_private_key)
     if signature_certificate_list is None:
       signature_certificate_list = config.get('signature-certificate-list')
     if type(signature_certificate_list) is str:
@@ -216,9 +220,18 @@ class NetworkcacheClient(object):
     for certificate in signature_certificate_list or ():
       try:
         loaded_certificate = crypto.load_certificate(crypto.FILETYPE_PEM, certificate)
-        self.signature_certificate_list.append(loaded_certificate)
       except Exception as e:
         logger.info('Ignored wrong certificate, reason:\n%s, offending certificate:\n%s', e, certificate)
+      else:
+        if pubkey and pubkey == crypto.dump_publickey(
+            crypto.FILETYPE_PEM, loaded_certificate.get_pubkey()):
+          pubkey = None
+        self.signature_certificate_list.append(loaded_certificate)
+    if pubkey:
+      raise NetworkcacheException(
+        'Invalid configuration: no matching certificate for private key'
+        ' (continuing would cause duplicate uploads because it would never'
+        ' manage to download these uploads)')
 
   # NetworkcacheClient context manager catches all exceptions and logs them
   # with INFO severity. This provides a easy way to use a networkcache safely
@@ -286,6 +299,25 @@ class NetworkcacheClient(object):
       connection.close()
     raise HTTPError(url, r.status, r.reason, r.msg, r.fp)
 
+  def tryDownload(self, key):
+    """
+    To be called before attemting to download from network cache.
+    bool(returned value) is True if that possible (i.e. configured),
+    - in which case the method logs what's going to happen;
+    - else the caller should continue without network cache.
+    """
+    config_get = self.config.get
+    if config_get('download-dir-url') and config_get('download-cache-url'):
+      logger.info('Trying to download %s from network cache...', key)
+      return True
+
+  def tryUpload(self, key):
+    """Similar to tryDownload but for upload"""
+    config_get = self.config.get
+    if config_get('upload-dir-url') and config_get('upload-cache-url'):
+      logger.info('Trying to upload %s to network cache...', key)
+      return True
+
   @staticmethod
   def archive(path):
     # Don't create it to /tmp dir as it can be too small.
@@ -311,8 +343,7 @@ class NetworkcacheClient(object):
     finally:
       f is None or f.close()
 
-  def upload(self, file_descriptor, key=None, urlmd5=None, file_name=None,
-                                    valid_until=None, architecture=None, **kw):
+  def upload(self, file_descriptor, key=None, **kw):
     ''' Upload the file to the server.
     If key is None it must only upload to SHACACHE.
     Otherwise, it must create a new entry on SHADIR.
@@ -355,16 +386,9 @@ class NetworkcacheClient(object):
       f is None or f.close()
 
     if key is not None:
-      kw['sha512'] = sha512sum # always update sha512sum
-      file_name = kw.pop('file', file_name)
-      if file_name is None or urlmd5 is None:
-        raise NetworkcacheException(
-          'file_name and urlmd5 are required for non-generic upload')
-      if valid_until is not None:
-        kw['valid-until'] = valid_until
-      if architecture is not None:
-        kw['architecture'] = architecture
-      self.index(key, file=file_name, urlmd5=urlmd5, **kw)
+      if not kw:
+          raise NetworkcacheException('metadata should not be empty')
+      self.index(key, sha512=sha512sum, **kw)
 
     return sha512sum
 
@@ -421,7 +445,7 @@ class NetworkcacheClient(object):
       raise NetworkcacheException('Failed to parse json response (%r)' % data)
     if filter:
       return (data for data in data_list
-                   if self._verifySignatureInCertificateList(*data))
+                   if self.verifySignatureInCertificateList(*data))
     return data_list
 
   def _getSignatureString(self, content):
@@ -433,7 +457,7 @@ class NetworkcacheClient(object):
         return ''
     return b64encode(crypto.sign(k, content, 'sha1')).decode()
 
-  def _verifySignatureInCertificateList(self, content, signature_string):
+  def verifySignatureInCertificateList(self, content, signature_string):
     """
       Returns true if it can find any valid certificate or false if it does not
       find any.
@@ -455,6 +479,8 @@ class NetworkcacheException(Exception):
 
 DirectoryNotFound = UploadError = NetworkcacheException # BBB
 
+key_help = \
+  " The key will be concatenation of PREFIX_KEY, md5(URL) and SUFFIX_KEY."
 
 def _newArgumentParser(url_help):
   parser = argparse.ArgumentParser()
@@ -462,15 +488,14 @@ def _newArgumentParser(url_help):
     help='SlapOS configuration file.')
   parser.add_argument('--prefix-key', default='')
   parser.add_argument('--suffix-key', default='')
-  parser.add_argument('--url', help=url_help
-    + " The key will be concatenation of PREFIX_KEY, md5(URL) and SUFFIX_KEY."
-    " If not given, the uploaded data is not indexed.")
+  parser.add_argument('--url', help=url_help)
   return parser
 
 def cmd_upload(*args):
-  parser = _newArgumentParser("Upload data pointed to by this argument,"
-    " unless --file is specified. If argument is not a local path, contents"
-    " is first downloaded in a temporary file.")
+  parser = _newArgumentParser(
+    "Upload data pointed to by this argument, unless --file is specified."
+    " Non-local contents is first downloaded to a temporary file."
+    "%s If not given, the uploaded data is not indexed." % key_help)
   parser.add_argument('--file',
     help="Upload the contents of this file, overriding --url")
   parser.add_argument('meta', nargs='*', metavar='KEY=VALUE',
@@ -485,20 +510,18 @@ def cmd_upload(*args):
         nc.upload(f)
         return
     elif args.url:
-      parser.error('either --file or --url is required')
-    elif os.path.isdir(args.url):
-      f = nc.archive(args.url)
+      f = (nc.archive if os.path.isdir(args.url) else urlopen)(args.url)
     else:
-      f = urlopen(args.url)
+      parser.error('either --file or --url is required')
+    kw = dict(x.split('=', 1) for x in args.meta)
+    kw.setdefault('url', args.url)
     urlmd5 = hashlib.md5(args.url.encode()).hexdigest()
-    nc.upload(f, args.prefix_key + urlmd5 + args.suffix_key, urlmd5=urlmd5,
-              file_name=os.path.basename(args.url) or "file",
-              **dict(x.split('=', 1) for x in args.meta))
+    nc.upload(f, args.prefix_key + urlmd5 + args.suffix_key, **kw)
   finally:
     f is None or f.close()
 
 def cmd_download(*args):
-  parser = _newArgumentParser("URL of data to download.")
+  parser = _newArgumentParser("URL of data to download." + key_help)
   args = parser.parse_args(args or sys.argv[1:])
   nc = NetworkcacheClient(args.config)
   urlmd5 = hashlib.md5(args.url.encode()).hexdigest()
